package com.hzya.frame.plugin.lets.plugin.base;

import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.plugin.lets.constant.OverallConstant;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;

import com.hzya.frame.plugin.lets.queryvo.CustdocRequestVo;

import com.hzya.frame.plugin.lets.queryvo.CustmandocRequestVo;
import com.hzya.frame.plugin.lets.resultvo.CbdocResultVo;
import com.hzya.frame.plugin.lets.u8cdto.CustdocEntityDto;
import com.hzya.frame.plugin.lets.u8cdto.CustmandocEntityDto;
import com.hzya.frame.plugin.lets.u8cdto.ReusltStrDto;
import com.hzya.frame.plugin.lets.util.PushDataByU8cUtil;
import com.hzya.frame.plugin.lets.util.SaveOrUpdateBusinessLogUtil;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @Author
 * @Date 2024/8/12 15:25
 * @PackageName:com.hzya.frame.plugin.lets.plugin.base
 * @ClassName: CustmandocPluginInitializer
 * @Description: 客商管理档案
 */
public class CustmandocPluginInitializer extends PluginBaseEntity {

    Logger logger = LoggerFactory.getLogger(CustmandocPluginInitializer.class);

    private static final ReentrantLock LOCK = new ReentrantLock(true);

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "com.hzya.frame.plugin.lets.plugin.base.CustmandocPluginInitializer";
    }

    @Override
    public String getPluginName() {
        return "丽知：OFS客商档案--->U8C客商管理档案";
    }

    @Override
    public String getPluginLabel() {
        return "丽知：U8C客商基本档案--->U8C客商管理档案";
    }

    @Override
    public String getPluginType() {
        return "1";
    }

    @Autowired
    private PushDataByU8cUtil pushDataByU8cUtil;
    @Autowired
    private IIntegrationTaskLivingDetailsDao iIntegrationTaskLivingDetailsDao;
    @Autowired
    private SaveOrUpdateBusinessLogUtil saveOrUpdateBusinessLogUtil;

    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        return null;
    }


    /**
     * 默认推送
     */
    public void start() {
        try {
            //获取符合条件客商基本档案
            List<CustdocRequestVo> custdocRequestVos = queryCustDoc();
            implement(custdocRequestVos);
        } catch (Exception e) {
            logger.error("丽知：U8C客商基本档案 --->U8C客商管理档案 ,start()方法报错：", e);
        }
    }

    /**
     * 查询客商基本档案
     * @return
     */
    private List<CustdocRequestVo> queryCustDoc() {
        List<CustdocRequestVo> custdocRequestVos=null;
        try {
            String data = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("custdocQuery"), "{}");
            CbdocResultVo cbdocResultVo = JSONObject.parseObject(data, CbdocResultVo.class);
            String datastring = cbdocResultVo.getData();
            Map map = JSONObject.parseObject(datastring, Map.class);
            Object o = map.get("datas");
            List<CustdocRequestVo> list = JSONObject.parseArray(JSON.toJSONString(o), CustdocRequestVo.class);
            custdocRequestVos = filterData(list);

        } catch (Exception e) {
            logger.error("丽知：查询客商基本档案失败", e);
        }
        return custdocRequestVos;
    }

    private List<CustdocRequestVo> filterData(List<CustdocRequestVo> list) {
        List<CustdocRequestVo> filterlist = new ArrayList<>();
        for (CustdocRequestVo requestVo : list) {
            if("N".equals(requestVo.getParentvo().getDef11())){
                filterlist.add(requestVo);
            }
        }
        return filterlist;
    }

    public void implement(List<CustdocRequestVo> list) {
        for (CustdocRequestVo vo : list) {
            String mandocjsonString = "";
            //生成业务日期

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"); // 定义日期格式
            Date date = new Date(System.currentTimeMillis()); // 创建Date对象
            String businessFormat = sdf.format(date); // 将Date对象格式化为字符串

            CustdocEntityDto parentvo = vo.getParentvo();
            try {

                parentvo.setPk_areacl("F");
                CustmandocRequestVo custmandocRequestVo = new CustmandocRequestVo();
                custmandocRequestVo.setParentvo(parentvo);
                custmandocRequestVo.setAddrs(vo.getAddrs());
//                custmandocRequestVo.setBanks(vo.getBanks());

                CustmandocEntityDto custmandocEntityDto = new CustmandocEntityDto();
                custmandocEntityDto.setCustflag("0");
                CustmandocEntityDto custmandocEntityDto1 = new CustmandocEntityDto();
                custmandocEntityDto1.setCustflag("4");
                List<CustmandocEntityDto> custmandocEntities = new ArrayList<>();
                custmandocEntities.add(custmandocEntityDto);
                custmandocEntities.add(custmandocEntityDto1);
                custmandocRequestVo.setChildrenvo(custmandocEntities);
                HashMap<String, Object> mandocparam = new HashMap<>();
                mandocparam.put("billvo", custmandocRequestVo);
                mandocjsonString = JSONObject.toJSONString(mandocparam);
                String response = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("custmandocSave"), mandocjsonString);
                System.out.println(response);
                boolean isSuccess = false;
                if (response != null && !"".equals(response)) {
                    ReusltStrDto reusltStrDto = JSON.parseObject(response, ReusltStrDto.class);
                    if ("success".equals(reusltStrDto.getStatus())) {
                        isSuccess = true;
                        updateDoc(vo);
                    }
                }
                if (!isSuccess) {
                    Assert.state(false, "推送U8C-->丽知：U8C客商基本档案 --->U8C客商管理档案 失败 接口返回结果：{} 接口入参:{}", response, mandocjsonString);
                }
                //成功
                IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
                integrationTaskLivingDetailsEntity.setRootAppNewData(mandocjsonString);
                integrationTaskLivingDetailsEntity.setNewTransmitInfo(response);
                integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                integrationTaskLivingDetailsEntity.setBusinessDate(businessFormat);
                integrationTaskLivingDetailsEntity.setRootAppPk(parentvo.getPk_cubasdoc());
                integrationTaskLivingDetailsEntity.setRootAppBill(parentvo.getCustcode());
                integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);


            } catch (Exception e) {
                e.printStackTrace();
                logger.error("丽知：U8C客商基本档案 --->U8C客商管理档案 ,implement()方法报错", e);
                //失败
                String ErrMessage = e.getMessage();
                IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
                integrationTaskLivingDetailsEntity.setRootAppNewData(mandocjsonString);
                integrationTaskLivingDetailsEntity.setNewTransmitInfo(ErrMessage);
                integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                integrationTaskLivingDetailsEntity.setBusinessDate(businessFormat);
                integrationTaskLivingDetailsEntity.setRootAppPk(parentvo.getPk_cubasdoc());
                integrationTaskLivingDetailsEntity.setRootAppBill(parentvo.getCustcode());
                integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
            }
        }
    }

    private void updateDoc(CustdocRequestVo custdocRequestVo) {
        //已分配
        String pkCubasdoc = custdocRequestVo.getParentvo().getPk_cubasdoc();
        String createtime = custdocRequestVo.getParentvo().getCreatetime();

        custdocRequestVo.getParentvo().setDef11("Y");
        ArrayList<CustdocRequestVo> requestVos = new ArrayList<>();
        requestVos.add(custdocRequestVo);
        HashMap<String, List> map = new HashMap<>();
        map.put("custbasvo", requestVos);
        String data = JSONObject.toJSONString(map);
        try {
            String response = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("custdocUpdate"), data);

            boolean isSuccess = false;
            if (response != null && !"".equals(response)) {
                ReusltStrDto reusltStrDto = JSON.parseObject(response, ReusltStrDto.class);
                if ("success".equals(reusltStrDto.getStatus())) {
                    isSuccess = true;
                }
            }
            if (!isSuccess) {
                Assert.state(false, "更新客商基本档案失败 接口返回结果：{} 接口入参:{}", response, data);
            }
            //成功
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
            integrationTaskLivingDetailsEntity.setRootAppNewData(data);
            integrationTaskLivingDetailsEntity.setNewTransmitInfo(response);
            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
            integrationTaskLivingDetailsEntity.setBusinessDate(createtime);
            integrationTaskLivingDetailsEntity.setRootAppPk(pkCubasdoc);
            integrationTaskLivingDetailsEntity.setRootAppBill(pkCubasdoc);
            integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);

        } catch (Exception e) {
            //失败
            logger.error("U8C->客商基本档案更新updateDoc(CustdocRequestVo custdocRequestVo)方法失败");
            String ErrMessage = e.getMessage();
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
            integrationTaskLivingDetailsEntity.setRootAppNewData(data);
            integrationTaskLivingDetailsEntity.setNewTransmitInfo(ErrMessage);
            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
            integrationTaskLivingDetailsEntity.setBusinessDate(createtime);
            integrationTaskLivingDetailsEntity.setRootAppPk(pkCubasdoc);
            integrationTaskLivingDetailsEntity.setRootAppBill(pkCubasdoc);
            integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
        }

    }
}
