package com.hzya.frame.plugin.lets.plugin.base;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.plugin.lets.constant.OverallConstant;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;
import com.hzya.frame.plugin.lets.entity.BdInvclEntity;
import com.hzya.frame.plugin.lets.plugin.sales.SoSaleOutPluginInitializerToC;
import com.hzya.frame.plugin.lets.u8cdto.ReusltStrDto;
import com.hzya.frame.plugin.lets.util.PushDataByU8cUtil;
import com.hzya.frame.plugin.lets.util.SaveOrUpdateBusinessLogUtil;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.web.entity.JsonResultEntity;
import com.zaxxer.hikari.util.FastList;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * OFS->U8C货品档案分类 新增，修改，删除
 * add by zyd 20240802
 *
 * 存货分类新增     /u8cloud/api/uapbd/bdinvcl/save
 * 存货分类修改     /u8cloud/api/uapbd/bdinvcl/update
 * 存货分类删除     /u8cloud/api/uapbd/bdinvcl/delete
 *
 * U8C分类规则 XX-XXX-XX，自动截断
 * eg：
 *  一级：AA
 *      二级：AA001,自动识别为AA的下级
 *          三级：AA00101自动识别为AA001的下级
 */
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.locks.ReentrantLock;

public class GoodsClassPluginInitializer extends PluginBaseEntity {

    Logger logger = LoggerFactory.getLogger(GoodsClassPluginInitializer.class);

    private static final ReentrantLock LOCK = new ReentrantLock(true);

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "com.hzya.frame.plugin.lets.plugin.base.GoodsClassPluginInitializer";
    }

    @Override
    public String getPluginName() {
        return "丽知：OFS货品档案分类--->U8C货品档案分类";
    }

    @Override
    public String getPluginLabel() {
        return "丽知：OFS货品档案分类--->U8C货品档案分类";
    }

    @Override
    public String getPluginType() {
        return "1";
    }

    @Autowired
    private PushDataByU8cUtil pushDataByU8cUtil;
    @Autowired
    private IIntegrationTaskLivingDetailsDao iIntegrationTaskLivingDetailsDao;
    @Autowired
    private SaveOrUpdateBusinessLogUtil saveOrUpdateBusinessLogUtil;

    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        return null;
    }


    /**
     * 默认推送
     */
    public void start() {
        try {
            //获取当前时间
            Date currentDate = new Date();
            Date startTime = DateUtil.offset(currentDate, DateField.MINUTE, -10);
            Date endTime = DateUtil.offset(currentDate, DateField.MINUTE, -2);
            String startTimeStr = DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss" + ".000");
            String endTimeStr = DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss" + ".000");
        } catch (Exception e) {
            logger.error("丽知：OFS货品档案分类--->U8C货品档案分类,start()方法报错：", e);
        }
    }

    /**
     * 按分类名称
     */
    public void start(String className) {
        try {
            implement();
        } catch (Exception e) {
            logger.error("丽知：OFS货品档案分类--->U8C货品档案分类,start(String className)方法报错：", e);
        }
    }

    /**
     * 按时间区间
     */
    public void start(String startTime, String endTime) {
        try {
            Date business_start = DateUtil.parse(startTime);
            Date business_end = DateUtil.parse(endTime);
            String start = DateUtil.format(business_start, "yyyy-MM-dd") + " 00:00:00";
            String end = DateUtil.format(business_end, "yyyy-MM-dd") + " 23:59:59";
        } catch (Exception e) {
            logger.error("丽知：OFS货品档案分类--->U8C货品档案分类,start(String startTime, String endTime)方法报错：", e);
        }
    }

    public void filterGoodsClass() {
        return;
    }

    /**
     * U8C分类规则 XX-XXX-XX，自动截断
     * eg：
     *  一级：AA
     *      二级：AA001,自动识别为AA的下级
     *          三级：AA00101自动识别为AA001的下级
     */
    public void implement() {

        //生成业务日期
        String businessFormat = "";
        String mapStr="";


        String invclasscode=null;
        String invclassname=null;
        try {

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"); // 定义日期格式
            Date date = new Date(System.currentTimeMillis()); // 创建Date对象
            businessFormat = sdf.format(date); // 将Date对象格式化为字符串




            List<BdInvclEntity> bdInvclEntities = new ArrayList<>();

            BdInvclEntity bdInvclEntity = new BdInvclEntity();

            //分类编码-invclasscode
            invclasscode = "AA004";
            bdInvclEntity.setInvclasscode(invclasscode);

            //分类名称-invclassname
            invclassname = "分类名称：AA004";

            bdInvclEntity.setInvclassname(invclassname);
            //公司编码-pk_corp
            String pk_corp = "1001";
            bdInvclEntity.setPk_corp(pk_corp);
            //分类级别-invclasslevel
            bdInvclEntities.add(bdInvclEntity);

            String jsonStr = JSONUtil.toJsonStr(bdInvclEntities);

            Map<String, Object> map = new HashMap<>();
            map.put("invcl", jsonStr);
            mapStr = JSONUtil.toJsonStr(map);

            String response = pushDataByU8cUtil.pushU8CByPK(OverallConstant.getOverAllValue("bdinvclSave"), mapStr);
            System.out.println(response);
            boolean isSuccess = false;
            if (response != null && !"".equals(response)) {
                ReusltStrDto reusltStrDto = JSON.parseObject(response, ReusltStrDto.class);
                if ("success".equals(reusltStrDto.getStatus())) {
                    isSuccess = true;
                }
            }
            if (!isSuccess) {
                Assert.state(false, "推送U8C-->丽知：OFS货品档案分类--->U8C货品档案分类失败 接口返回结果：{} 接口入参:{}", response, jsonStr);
            }
            //成功
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
            integrationTaskLivingDetailsEntity.setRootAppNewData(mapStr);
            integrationTaskLivingDetailsEntity.setNewTransmitInfo(response);
            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
            integrationTaskLivingDetailsEntity.setBusinessDate(businessFormat);
            integrationTaskLivingDetailsEntity.setRootAppPk(invclasscode);
            integrationTaskLivingDetailsEntity.setRootAppBill(invclassname);
            integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);


        } catch (Exception e) {
            e.printStackTrace();
            logger.error("丽知：OFS货品档案分类--->U8C货品档案分类,implement()方法报错：", e);
            //失败
            String ErrMessage = e.getMessage();
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
            integrationTaskLivingDetailsEntity.setRootAppNewData(mapStr);
            integrationTaskLivingDetailsEntity.setNewTransmitInfo(ErrMessage);
            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
            integrationTaskLivingDetailsEntity.setBusinessDate(businessFormat);
            integrationTaskLivingDetailsEntity.setRootAppPk(invclasscode);
            integrationTaskLivingDetailsEntity.setRootAppBill(invclassname);
            integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
        }

    }


}
