package com.hzya.frame.plugin.lets.plugin.base;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.plugin.lets.constant.OverallConstant;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;

import com.hzya.frame.plugin.lets.queryvo.CustdocRequestVo;
import com.hzya.frame.plugin.lets.resultvo.CbdocResultVo;
import com.hzya.frame.plugin.lets.resultvo.ShopReturnMessageDto;
import com.hzya.frame.plugin.lets.u8cdto.CustdocEntityDto;
import com.hzya.frame.plugin.lets.util.PushDataByU8cUtil;
import com.hzya.frame.plugin.lets.util.SaveOrUpdateBusinessLogUtil;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.ttxofs.basics.ReturnMessageBasics;
import com.hzya.frame.ttxofs.dto.InterfaceParamByU8CApiDto;
import com.hzya.frame.ttxofs.dto.InterfaceParamDto;
import com.hzya.frame.ttxofs.dto.ofsBaseSearch.OfsShopDto;
import com.hzya.frame.ttxofs.dto.ofsBaseSearch.OfsShopReturnMessageDto;
import com.hzya.frame.ttxofs.service.OfsUnifiedService;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @Author
 * @Date 2024/8/6 14:36
 * @PackageName:com.hzya.frame.plugin.lets.plugin.base
 * @ClassName: ShopPluginInitializer
 * @Description: 店铺->U8C客商档案
 */

public class ShopPluginInitializer extends PluginBaseEntity {
    Logger logger = LoggerFactory.getLogger(ShopPluginInitializer.class);

    private static final ReentrantLock LOCK = new ReentrantLock(true);

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "执行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "执行销毁方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "ShopPluginInitializer";
    }

    @Override
    public String getPluginName() {
        return "丽知：OFS店铺档案--->U8C客商档案";
    }

    @Override
    public String getPluginLabel() {
        return "丽知：OFS店铺档案--->U8C客商档案";
    }

    @Override
    public String getPluginType() {
        return "1";
    }


//    @Autowired
//    private IShopDao shopDao;

    @Autowired
    private SaveOrUpdateBusinessLogUtil saveOrUpdateBusinessLogUtil;
    @Autowired
    private IIntegrationTaskLivingDetailsDao iIntegrationTaskLivingDetailsDao;

    @Autowired
    private PushDataByU8cUtil pushDataByU8cUtil;

    @Autowired
    private OfsUnifiedService ofsUnifiedService;


    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        String param = String.valueOf(requestJson.get("param"));
        if (requestJson != null && ProfilesActiveConstant.TYPE_DATE.equals(requestJson.get("type"))) {
            if (param != null && !"".equals(param)) {
                String[] split = param.split("/");
                startImplement(split[0], split[1]);
            }
        } else if (requestJson != null && ProfilesActiveConstant.TYPE_VBILLCODE.equals(requestJson.get("type"))) {
            if (param != null && !"".equals(param)) {
                startImplement(param);
            }
        } else if (ProfilesActiveConstant.TYPE_DETAIL_ERROR.equals(requestJson.get("type"))) {
            String integration_task_living_details_id = (String) requestJson.get("integration_task_living_details_id");
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = saveOrUpdateBusinessLogUtil.queryIntegrationTaskLivingDetailsEntity(integration_task_living_details_id);
            startImplement(integrationTaskLivingDetailsEntity.getRootAppPk());
        } else {
            startImplement();
        }
        return null;
    }


    public void startImplement(String startDate, String endDate) {
        try {
            OfsShopDto shopEntity = new OfsShopDto();

            InterfaceParamDto interfaceParamDto = new InterfaceParamDto();
            interfaceParamDto.setApi("ofs.store.search");
            interfaceParamDto.setData(JSON.toJSONString(shopEntity));
            List<OfsShopDto> list = queryfromofs(shopEntity);


            List<OfsShopDto> shopEntities = filterShopEntity(list);
            implement(shopEntities);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 默认推送
     */
    public void startImplement() {
        try {
            //获取当前时间
            Date currentDate = new Date();
            Date startTime = DateUtil.offset(currentDate, DateField.MINUTE, -10);
            Date endTime = DateUtil.offset(currentDate, DateField.MINUTE, -2);
            String startTimeStr = DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss");
            String endTimeStr = DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss");


            OfsShopDto shopEntity = new OfsShopDto();


            //TODO ofs查询店铺信息
//            shopEntity.setLastUpdated_start(startTimeStr);
//            shopEntity.setLastUpdated_end(endTimeStr);
            shopEntity.setPageNo("1");
            shopEntity.setClientCode("LETS");
            List<OfsShopDto> list = queryfromofs(shopEntity);

            //过滤数据
            List<OfsShopDto> ofsShopDtoList = filterShopEntity(list);
            //同步u8c
            implement(ofsShopDtoList);

        } catch (Exception e) {
            logger.error("丽知：OFS货品档案分类--->U8C货品档案分类,startImplement()方法报错：", e);
        }
    }


    /**
     * 按编码名称
     */
    public void startImplement(String clientcode) {
        Assert.notNull(clientcode, "clientcode不能为空");
        try {
            OfsShopDto shopEntity = new OfsShopDto();
            shopEntity.setClientCode(clientcode);
            shopEntity.setPageNo("1");
            //TODO ofs查询店铺信息
            List<OfsShopDto> list = queryfromofs(shopEntity);
            //过滤数据
            List<OfsShopDto> shopEntities = filterShopEntity(list);
            //执行
            implement(shopEntities);

        } catch (Exception e) {
            logger.error("丽知：OFS店铺档案--->U8C客商档案,startImplement(String code)方法报错：", e);
        }
    }

    @Transactional
    public void implement(List<OfsShopDto> shopEntityList) {
        Assert.notNull(shopEntityList, "shopEntityList不能为空");

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = new Date(System.currentTimeMillis());
        String businessFormat = sdf.format(date);
        for (OfsShopDto shopEntity : shopEntityList) {

            try {

                CustdocEntityDto parentvo = new CustdocEntityDto();
                CustdocRequestVo custdocRequestVo = new CustdocRequestVo();
                //TODO 设置parentvo
                parentvo.setCustname(shopEntity.getName());
                parentvo.setCustshortname(shopEntity.getName());
//                parentvo.setCustcode(shopEntity.getCustomerCode());
//                parentvo.setCustcode(shopEntity.getBizCode());
//                parentvo.setPk_areacl("F");
                //设置还未分配公司
//                parentvo.setDef10("N");
//                //设置还未同步到管理档案
//                parentvo.setDef11("N");
//                parentvo.setCustprop("2");
                custdocRequestVo.setParentvo(parentvo);

                //同步u8c
                HashMap<String, Object> param = new HashMap<>();
                param.put("cbdocvo", custdocRequestVo);
                String jsonString = JSONObject.toJSONString(param);
                String response = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("custdocSave"), jsonString);
                System.out.println(response);
                if (response != null) {
                    CbdocResultVo cbdocResultVo = JSONObject.parseObject(response, CbdocResultVo.class);
                    String taskNumber = null;
                    boolean isSuccess = false;
                    if ("success".equals(cbdocResultVo.getStatus())) {
                        isSuccess = true;
                        taskNumber = cbdocResultVo.getTaskNumber();
                        String data = cbdocResultVo.getData();
                    }

                    if (!isSuccess) {
                        Assert.state(false, "同步到U8C失败，原因：{} 接口入参:{} {}", JSON.toJSONString(cbdocResultVo), JSON.toJSONString(OverallConstant.getOverAllValue("custdocSave")), JSON.toJSONString(custdocRequestVo));
                    }
                    // 根据返回结果记录日志
                    //成功
                    IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                    integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
                    integrationTaskLivingDetailsEntity.setRootAppNewData(jsonString);
                    integrationTaskLivingDetailsEntity.setNewTransmitInfo(JSON.toJSONString(cbdocResultVo));
                    integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                    integrationTaskLivingDetailsEntity.setBusinessDate(businessFormat);
                    integrationTaskLivingDetailsEntity.setRootAppPk(shopEntity.getBizCode());
                    integrationTaskLivingDetailsEntity.setRootAppBill(shopEntity.getName());
                    integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                    saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
                    //TODO 更新店铺状态
//                    CbdocResultVo cbdocResultVo1 = JSONObject.parseObject(response, CbdocResultVo.class);
//                    String errormsg = cbdocResultVo1.getErrormsg();
                }

            } catch (Exception e) {
                e.printStackTrace();
                String ErrMessage = e.getMessage();
                IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
                integrationTaskLivingDetailsEntity.setRootAppNewData(JSONObject.toJSONString(shopEntity));
                integrationTaskLivingDetailsEntity.setNewTransmitInfo(ErrMessage);
                integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                integrationTaskLivingDetailsEntity.setBusinessDate(businessFormat);
                integrationTaskLivingDetailsEntity.setRootAppPk(shopEntity.getBizCode());
                integrationTaskLivingDetailsEntity.setRootAppBill(shopEntity.getName());
                integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
                logger.error("丽知：OFS店铺档案--->U8C客商档案,implement()方法报错：", e);
            }
        }


    }

    public List<OfsShopDto> filterShopEntity(List<OfsShopDto> list) {
        Assert.notNull(list, "不能为空！");
        List<OfsShopDto> shopEntityList = new ArrayList<>();
        if (list != null && list.size() > 0) {
            for (int i = 0; i < list.size(); i++) {
                OfsShopDto ofsShopDto = list.get(i);

                if (ofsShopDto.getBizCode() != null && !"".equals(ofsShopDto.getBizCode().trim())) {
                    boolean isExis = true;
                    IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                    integrationTaskLivingDetailsEntity.setRootAppPk(ofsShopDto.getBizCode());
                    integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y_H);
                    integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                    List<IntegrationTaskLivingDetailsEntity> integrationTaskLivingDetailsEntities = iIntegrationTaskLivingDetailsDao.query(integrationTaskLivingDetailsEntity);
                    if (integrationTaskLivingDetailsEntities == null || integrationTaskLivingDetailsEntities.size() == 0) {
                        isExis = false;
                    }
                    if (!isExis) {
                        shopEntityList.add(ofsShopDto);
                    }
                }
            }
        }
        return shopEntityList;

    }

    public List<OfsShopDto> queryfromofs(OfsShopDto shopEntity) {
        InterfaceParamByU8CApiDto interfaceParamDto = new InterfaceParamByU8CApiDto();
        interfaceParamDto.setApi("ofs.store.search");
        interfaceParamDto.setData(JSON.toJSONString(shopEntity));
        try {
            OfsShopReturnMessageDto unified = (OfsShopReturnMessageDto) ofsUnifiedService.unifiedByU8CApi(interfaceParamDto);
            List<OfsShopDto> list = unified.getData();
            return list;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
